<?php
// Survey database interface class as intermediate for storing data from the site to the RDF database 
require_once 'rdfConstants.php';
// Include RAP Library to write RDF files
include(RDFAPI_INCLUDE_DIR . "RDFAPI.php");

/**
 * Description of Question
 *
 * @author jkraaijeveld
 */
class Question extends ResearchToolObject{    
	private static $filename = 'data/questions/questions.rdf';

    public $title;
    public $type;
    public $description;
    public $category;
    public $answers; // format answers['#']
    
    /**
     * Constructor for a Question. $uid equals the corresponding code.
     * @param type $uid
     * @param type $title
     * @param type $type
     * @param type $description
     * @param type $category
     * @param type $answers 
     */
    public function __construct($uid, $title = null, $type = null, $description = null, $category = null, $answers = null)
    {
		if(!isset($uid))
		{
			$uid = md5(uniqid(rand(), true));
		}
        $this->uid = $uid;
        $this->title = $title;
        $this->type = $type;
        $this->description = $description;
        $this->category =  $category;
        $this->answers = $answers;
	}

	/**
	 * function save()
	 * Saves the current object into the database.
	 */
	public function save()
	{
        //Ensure the required folder exists
        if (!is_dir('data/questions/'))
            mkdir('data/questions/');	

		$model = ResearchToolObject::load(Question::$filename);
		
		$resourceQuestion = new Resource(QUESTION.'/'.$this->uid);
		//Remove the old value stored with the given id
        $model->subtract($model->find($resourceQuestion, null, null));
        
        //Save all the new values
        $resourceQuestionType = new Resource(QUESTION);
        $predicateRType = new Resource(RTYPE);
        $model->add(new Statement($resourceQuestion,$predicateRType,$resourceQuestionType));
        
        $questionQCode = new Literal($this->uid);
        $predicateQCode = new Resource(QCODE);
        $model->add(new Statement($resourceQuestion,$predicateQCode,$questionQCode));
        
        $questionTitle = new Literal($this->title);
        $predicateTitle = new Resource(TITLE);		
        $model->add(new Statement($resourceQuestion,$predicateTitle,$questionTitle));	

        $questionDescription = new Literal($this->description);
        $predicateDescription = new Resource(DESCRIPTION);
        $model->add(new Statement($resourceQuestion,$predicateDescription,$questionDescription));		

        $resourceQuestionType = new Literal($this->type);
        $predicateQType = new Resource(QTYPE);
        $model->add(new Statement($resourceQuestion,$predicateQType,$resourceQuestionType));
        
        $questionQCategory = new Literal($this->category);
        $predicateQCategory = new Resource(QCATEGORY);
        $model->add(new Statement($resourceQuestion,$predicateQCategory,$questionQCategory));

        if(isset($this->answers))
        {
            foreach($this->answers as $answer)
            {		
                $answerValue = new Literal($answer);
                $predicateAnswer = new Resource(HAS_ANSWER);	
                $model->add(new Statement($resourceQuestion,$predicateAnswer,$answerValue));
            }
        }
		$model->saveAs(Question::$filename, 'rdf');
		return true;
	}

    /**
     * Get the questions corresponding to the arguments.
	 * @param type $arguments : An array having one ore more of the following elements:
	 * 'code', 'title', 'type', 'description', 'category', 'definedanswers'.
     */
    public static function get($arguments)
    {
		$model = ResearchToolObject::load(Question::$filename);
           
        $querystring = '
            PREFIX  predicates: <' . SURVEYTOOL_PREDICATES_NAMESPACE . '>
            PREFIX  resources: <' . SURVEYTOOL_RESOURCES_NAMESPACE . '>
            SELECT DISTINCT ?uid, ?title, ?type, ?description, ?category
            WHERE 	
            {
                _question	predicates:resource_type	resources:question ;                
                predicates:question_code ?uid ;
                predicates:title ?title ;
                predicates:question_type ?type ;
                predicates:description ?description ;
                predicates:question_category ?category ;
				'. ResearchToolObject::createArguments($arguments) . '
	}';

		
        //Create the querystring 
        $results = $model->sparqlQuery($querystring);

        
        $questions = array();
        if(!empty($results))
        {
            foreach($results as $result)
            {
				$answers = Question::getAnswers($model, $result['?uid']->label);
                $questions[] = new Question($result['?uid']->label, $result['?title']->label, $result['?type']->label, $result['?description']->label, $result['?category']->label, $answers);
            }
        }
        return $questions;
    }
    
    /**
     * Gets the answers belonging to the given uid.
     * @param type $uid : The uid of the Question in question (haha, pun).
     */
    private static function getAnswers($model, $uid)
    {
		$result = $model->findRegex("[(".$uid.")]", "[(has_answer)]", null);
		$iterator = $result->getStatementIterator();
		$answers = array();
		while($iterator->hasNext())
		{
			$element = $iterator->next();
			$answers[] = $element->getLabelObject();
		}
		return $answers;
	}

}

?>
